import { needRecursive, areEqual } from "./utils";
function computeOperation(before, after) {
    if (before === undefined && after !== undefined) {
        return "add";
    }
    else if (before !== undefined && after === undefined) {
        return "delete";
    }
    else if (!areEqual(before, after)) {
        return "update";
    }
    else {
        return "same";
    }
}
function getKeys(x, y) {
    return Array.from(new Set(Object.keys(x).concat(Object.keys(y))));
}
function makeDiff(before, after, path) {
    const operation = computeOperation(before, after);
    if (operation === "same")
        return [];
    return [
        {
            operation,
            path: path.length ? path.join(".") : undefined,
            before: operation !== "add" ? before : undefined,
            after: operation === "add" || operation === "update" ? after : undefined,
        },
    ];
}
function diffRecursive(before, after, path) {
    if (needRecursive(before, after)) {
        return getKeys(before, after).reduce((acc, value) => acc.concat(diffRecursive(before[value], after[value], path.concat(value))), []);
    }
    else {
        return makeDiff(before, after, path);
    }
}
export function diff(before, after) {
    return diffRecursive(before, after, []);
}
const getValueByPathImpl = (obj, path) => {
    if (!path)
        return obj;
    const [key, ...rest] = path.split(".");
    return getValueByPathImpl(obj[key], rest.join("."));
};
export const getValueByPath = (obj, path) => getValueByPathImpl(obj, path);
//# sourceMappingURL=recursiveDiff.js.map