/**
 * Checks if a value is an iterable object.
 * @param x - The value to check.
 * @returns True if the value is an iterable object, false otherwise.
 */
export declare const isIterableObject: (x: unknown) => boolean;
/**
 * The native types in JavaScript.
 */
declare const nativeTypes: readonly ["number", "boolean", "string", "bigint", "symbol", "undefined", "function", "object"];
/**
 * The value object types in JavaScript.
 */
declare const valueObjectTypes: readonly ["null", "date"];
/**
 * The possible types in JavaScript.
 */
type Types = (typeof nativeTypes)[number] | (typeof valueObjectTypes)[number];
/**
 * Gets the type of a value.
 * @param value - The value to get the type of.
 * @returns The type of the value.
 */
export declare function getType(value: any): Types;
/**
 * Checks if two values are equal.
 * @param before - The first value.
 * @param after - The second value.
 * @returns True if the values are equal, false otherwise.
 */
export declare function areEqual(before: unknown, after: unknown): boolean;
/**
 * Checks if recursive traversal is needed for two values.
 * @param before - The first value.
 * @param after - The second value.
 * @returns True if recursive traversal is needed, false otherwise.
 */
export declare function needRecursive(before: any, after: any): boolean;
export {};
